// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/src/domain/entities/text_inputs/text_input_style.dart';
import 'package:wyatt_ui_components/src/domain/theme_extensions/text_input_theme_extension.dart';

/// {@template text_input_theme_extension_default}
/// The default implementation of [TextInputThemeExtension].
/// {@endtemplate}
class TextInputThemeExtensionDefault extends TextInputThemeExtension {
  /// {@macro text_input_theme_extension_default}
  const TextInputThemeExtensionDefault({
    required super.normalStyle,
    required super.focusedStyle,
    required super.disabledStyle,
    required super.invalidStyle,
  });

  /// Creates a [TextInputThemeExtensionDefault] from a [ThemeData].
  ///
  /// {@macro text_input_theme_extension_default}
  factory TextInputThemeExtensionDefault.from(ThemeData theme) {
    final labelStyle = theme.textTheme.labelLarge
        ?.copyWith(color: theme.unselectedWidgetColor);

    final hintStyle = theme.textTheme.labelLarge;
    final prefixStyle = theme.textTheme.bodyMedium;
    final suffixStyle = theme.textTheme.bodyMedium;
    final inputStyle = theme.textTheme.bodyMedium;

    final iconColor = theme.colorScheme.inversePrimary;
    final prefixIconColor = theme.unselectedWidgetColor;
    final suffixIconColor = theme.unselectedWidgetColor;

    final borderColors = theme.unselectedWidgetColor;

    final style = TextInputStyle(
      labelStyle: labelStyle,
      hintStyle: hintStyle,
      iconColor: iconColor,
      prefixIconColor: prefixIconColor,
      prefixStyle: prefixStyle,
      suffixStyle: suffixStyle,
      suffixIconColor: suffixIconColor,
      borderColors: borderColors,
      inputStyle: inputStyle,
    );

    return TextInputThemeExtensionDefault(
      normalStyle: style,
      focusedStyle: style.copyWith(
        prefixIconColor: theme.colorScheme.primary,
        suffixIconColor: theme.colorScheme.primary,
        iconColor: theme.colorScheme.primary,
        borderColors: theme.colorScheme.primary,
        labelStyle: labelStyle?.copyWith(color: theme.colorScheme.primary),
      ),
      disabledStyle: style.copyWith(
        labelStyle: labelStyle?.copyWith(color: theme.disabledColor),
        hintStyle: hintStyle?.copyWith(color: theme.disabledColor),
        prefixStyle: prefixStyle?.copyWith(color: theme.disabledColor),
        suffixStyle: suffixStyle?.copyWith(color: theme.disabledColor),
        inputStyle: inputStyle?.copyWith(color: theme.disabledColor),
        borderColors: theme.disabledColor,
        prefixIconColor: theme.disabledColor,
        suffixIconColor: theme.disabledColor,
      ),
      invalidStyle: style.copyWith(
        labelStyle: theme.textTheme.labelLarge
            ?.copyWith(color: theme.colorScheme.error),
        borderColors: theme.colorScheme.error,
      ),
    );
  }

  /// Creates a [TextInputThemeExtensionDefault] from a [ThemeData.light].
  factory TextInputThemeExtensionDefault.light() =>
      TextInputThemeExtensionDefault.from(ThemeData.light());

  /// Creates a [TextInputThemeExtensionDefault] from a [ThemeData.dark].
  factory TextInputThemeExtensionDefault.dark() =>
      TextInputThemeExtensionDefault.from(ThemeData.dark());
}
