// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/wyatt_ui_kit.dart';

enum TextType {
  title,
  subtitle,
  body;
}

/// {@template card_text}
/// Wrapper for [GradientText] that uses the [CardThemeExtension] to
/// determine the text style to use depending on the [textType].
/// {@endtemplate}
class CardText extends StatelessWidget {
  /// {@macro card_text}
  const CardText(
    this.text, {
    this.textType = TextType.body,
    super.key,
  });

  /// Creates a [CardText] from a [TextWrapper].
  ///
  /// The [textType] is used to determine the [CardThemeExtension] style to use.
  ///
  /// You can also pass a [gradientColors] and [style] as fallbacks if the
  /// [TextWrapper] doesn't have them.
  factory CardText.fromWrapper(
    TextWrapper wrapper, {
    TextType textType = TextType.body,
    MultiColor? gradientColors,
    TextStyle? style,
  }) =>
      CardText(
        GradientText.fromWrapper(
          wrapper,
          gradientColors: gradientColors,
          style: style,
        ),
        textType: textType,
      );

  final GradientText text;
  final TextType textType;

  TextStyle? _getStyle(BuildContext context) {
    if (text.style != null) {
      return text.style;
    } else {
      final cardThemeExtension =
          Theme.of(context).extension<CardThemeExtension>() ??
              CardThemeExtensionDefault.from(Theme.of(context));
      switch (textType) {
        case TextType.title:
          return cardThemeExtension.titleStyle ??
              CardThemeExtensionDefault.from(Theme.of(context)).titleStyle;
        case TextType.subtitle:
          return cardThemeExtension.subtitleStyle ??
              CardThemeExtensionDefault.from(Theme.of(context)).subtitleStyle;
        case TextType.body:
          return cardThemeExtension.bodyStyle ??
              CardThemeExtensionDefault.from(Theme.of(context)).bodyStyle;
      }
    }
  }

  @override
  Widget build(BuildContext context) => text.copyWith
      .call(
        style: _getStyle(context),
      )
      .build(context);
}
